#ifndef _eval_h
#define _eval_h

#include <solution.h>

/******************************************************
 * Abstract class for the computation of bit string evaluation
 */
class Eval {
public:
  /** constructor
   * 
   * @param _n problem size, length of the bit string
   */
  Eval(unsigned int _n) : _pbSize(_n) { }

  // return the size of the problem
  unsigned int size() {
    return _pbSize;
  }

  // evaluation function to be defined
  virtual void operator()(Solution & _solution) = 0;

protected:
  // bit string length
  unsigned int _pbSize;
};

/******************************************************
  OneMax evaluation function:
  count the number of "1" in the bit string
  
  @param _solution the solution to evaluated
 */

class OneMax : public Eval {
public:
  // constructor
  OneMax(unsigned int _n) : Eval(_n) { }

  // evaluation function 
  void operator()(Solution & _solution) {
    int value = 0;

    for(unsigned int i = 0; i < _solution.size(); i++)
      if (_solution[i])
	value++;

    _solution.fitness(value);
  }
};

#endif

